@@ -1,6 +1,6 @@
 //
 //  ========================================================================
-//  Copyright (c) 1995-2020 Mort Bay Consulting Pty Ltd and others.
+//  Copyright (c) 1995-2022 Mort Bay Consulting Pty Ltd and others.
 //  ------------------------------------------------------------------------
 //  All rights reserved. This program and the accompanying materials
 //  are made available under the terms of the Eclipse Public License v1.0
@@ -26,27 +26,36 @@ import org.eclipse.jetty.client.api.Result;
 import org.eclipse.jetty.client.util.BufferingResponseListener;
 import org.eclipse.jetty.http.HttpHeader;
 import org.eclipse.jetty.http.HttpHeaderValue;
+import org.eclipse.jetty.http.HttpStatus;
 
+/**
+ * <p>A protocol handler that handles the 100 response code.</p>
+ */
 public class ContinueProtocolHandler implements ProtocolHandler
 {
+    public static final String NAME = "continue";
     private static final String ATTRIBUTE = ContinueProtocolHandler.class.getName() + ".100continue";
 
-    private final HttpClient client;
     private final ResponseNotifier notifier;
 
-    public ContinueProtocolHandler(HttpClient client)
+    public ContinueProtocolHandler()
     {
-        this.client = client;
         this.notifier = new ResponseNotifier();
     }
 
+    @Override
+    public String getName()
+    {
+        return NAME;
+    }
+
     @Override
     public boolean accept(Request request, Response response)
     {
+        boolean is100 = response.getStatus() == HttpStatus.CONTINUE_100;
         boolean expect100 = request.getHeaders().contains(HttpHeader.EXPECT, HttpHeaderValue.CONTINUE.asString());
-        HttpConversation conversation = ((HttpRequest)request).getConversation();
-        boolean handled100 = conversation.getAttribute(ATTRIBUTE) != null;
-        return expect100 && !handled100;
+        boolean handled100 = request.getAttributes().containsKey(ATTRIBUTE);
+        return (is100 || expect100) && !handled100;
     }
 
     @Override
@@ -56,6 +65,10 @@ public class ContinueProtocolHandler implements ProtocolHandler
         return new ContinueListener();
     }
 
+    protected void onContinue(Request request)
+    {
+    }
+
     protected class ContinueListener extends BufferingResponseListener
     {
         @Override
@@ -64,35 +77,31 @@ public class ContinueProtocolHandler implements ProtocolHandler
             // Handling of success must be done here and not from onComplete(),
             // since the onComplete() is not invoked because the request is not completed yet.
 
-            HttpConversation conversation = ((HttpRequest)response.getRequest()).getConversation();
+            Request request = response.getRequest();
+            HttpConversation conversation = ((HttpRequest)request).getConversation();
             // Mark the 100 Continue response as handled
-            conversation.setAttribute(ATTRIBUTE, Boolean.TRUE);
+            request.attribute(ATTRIBUTE, Boolean.TRUE);
 
             // Reset the conversation listeners, since we are going to receive another response code
             conversation.updateResponseListeners(null);
 
             HttpExchange exchange = conversation.getExchanges().peekLast();
-            assert exchange.getResponse() == response;
-            switch (response.getStatus())
+            if (response.getStatus() == HttpStatus.CONTINUE_100)
             {
-                case 100:
-                {
-                    // All good, continue
-                    exchange.resetResponse();
-                    exchange.proceed(null);
-                    break;
-                }
-                default:
-                {
-                    // Server either does not support 100 Continue,
-                    // or it does and wants to refuse the request content,
-                    // or we got some other HTTP status code like a redirect.
-                    List<Response.ResponseListener> listeners = exchange.getResponseListeners();
-                    HttpContentResponse contentResponse = new HttpContentResponse(response, getContent(), getMediaType(), getEncoding());
-                    notifier.forwardSuccess(listeners, contentResponse);
-                    exchange.proceed(new HttpRequestException("Expectation failed", exchange.getRequest()));
-                    break;
-                }
+                // All good, continue.
+                exchange.resetResponse();
+                exchange.proceed(null);
+                onContinue(request);
+            }
+            else
+            {
+                // Server either does not support 100 Continue,
+                // or it does and wants to refuse the request content,
+                // or we got some other HTTP status code like a redirect.
+                List<Response.ResponseListener> listeners = exchange.getResponseListeners();
+                HttpContentResponse contentResponse = new HttpContentResponse(response, getContent(), getMediaType(), getEncoding());
+                notifier.forwardSuccess(listeners, contentResponse);
+                exchange.proceed(new HttpRequestException("Expectation failed", request));
             }
         }
 
