@@ -1,6 +1,6 @@
 //
 //  ========================================================================
-//  Copyright (c) 1995-2021 Mort Bay Consulting Pty Ltd and others.
+//  Copyright (c) 1995-2022 Mort Bay Consulting Pty Ltd and others.
 //  ------------------------------------------------------------------------
 //  All rights reserved. This program and the accompanying materials
 //  are made available under the terms of the Eclipse Public License v1.0
@@ -28,6 +28,7 @@ import java.net.SocketTimeoutException;
 import java.nio.ByteBuffer;
 import java.nio.charset.StandardCharsets;
 import java.util.Arrays;
+import java.util.List;
 import java.util.concurrent.CountDownLatch;
 import java.util.concurrent.ExecutionException;
 import java.util.concurrent.Executor;
@@ -43,10 +44,13 @@ import javax.net.ssl.SSLSocket;
 
 import org.eclipse.jetty.client.api.ContentResponse;
 import org.eclipse.jetty.http.HttpHeader;
+import org.eclipse.jetty.http.HttpHeaderValue;
 import org.eclipse.jetty.http.HttpScheme;
 import org.eclipse.jetty.http.HttpStatus;
 import org.eclipse.jetty.io.ByteBufferPool;
 import org.eclipse.jetty.io.ClientConnectionFactory;
+import org.eclipse.jetty.io.Connection;
+import org.eclipse.jetty.io.ConnectionStatistics;
 import org.eclipse.jetty.io.EndPoint;
 import org.eclipse.jetty.io.ssl.SslClientConnectionFactory;
 import org.eclipse.jetty.io.ssl.SslConnection;
@@ -59,15 +63,16 @@ import org.eclipse.jetty.server.SecureRequestCustomizer;
 import org.eclipse.jetty.server.Server;
 import org.eclipse.jetty.server.ServerConnector;
 import org.eclipse.jetty.server.SslConnectionFactory;
+import org.eclipse.jetty.toolchain.test.Net;
 import org.eclipse.jetty.util.StringUtil;
 import org.eclipse.jetty.util.ssl.SslContextFactory;
 import org.eclipse.jetty.util.thread.ExecutorThreadPool;
 import org.eclipse.jetty.util.thread.QueuedThreadPool;
 import org.hamcrest.Matchers;
 import org.junit.jupiter.api.AfterEach;
-import org.junit.jupiter.api.Disabled;
+import org.junit.jupiter.api.Assumptions;
 import org.junit.jupiter.api.Test;
-import org.junit.jupiter.api.condition.EnabledOnJre;
+import org.junit.jupiter.api.condition.EnabledForJreRange;
 import org.junit.jupiter.api.condition.JRE;
 
 import static org.hamcrest.MatcherAssert.assertThat;
@@ -75,6 +80,7 @@ import static org.hamcrest.Matchers.instanceOf;
 import static org.junit.jupiter.api.Assertions.assertArrayEquals;
 import static org.junit.jupiter.api.Assertions.assertEquals;
 import static org.junit.jupiter.api.Assertions.assertNotNull;
+import static org.junit.jupiter.api.Assertions.assertNull;
 import static org.junit.jupiter.api.Assertions.assertThrows;
 import static org.junit.jupiter.api.Assertions.assertTrue;
 
@@ -260,10 +266,20 @@ public class HttpClientTLSTest
         assertTrue(clientLatch.await(1, TimeUnit.SECONDS));
     }
 
-    // In JDK 11+, a mismatch on the client does not generate any bytes towards
-    // the server, while in previous JDKs the client sends to the server the close_notify.
-    // @EnabledOnJre({JRE.JAVA_8, JRE.JAVA_9, JRE.JAVA_10})
-    @Disabled("No longer viable, TLS protocol behavior changed in 8u272")
+    /**
+     * This tests the behavior of the Client side when you have an intentional
+     * mismatch of the TLS Protocol and TLS Ciphers on the client and attempt to
+     * initiate a connection.
+     * <p>
+     * In older versions of Java 8 (pre 8u272) the JDK client side logic
+     * would generate bytes and send it to the server along with a close_notify
+     * </p>
+     * <p>
+     * Starting in Java 8 (8u272) and Java 11.0.0, the client logic will not
+     * send any bytes to the server.
+     * </p>
+     */
+    @Test
     public void testMismatchBetweenTLSProtocolAndTLSCiphersOnClient() throws Exception
     {
         SslContextFactory serverTLSFactory = createServerSslContextFactory();
@@ -280,11 +296,18 @@ public class HttpClientTLSTest
         });
 
         SslContextFactory clientTLSFactory = createClientSslContextFactory();
-        // TLS 1.1 protocol, but only TLS 1.2 ciphers.
-        clientTLSFactory.setIncludeProtocols("TLSv1.1");
-        clientTLSFactory.setIncludeCipherSuites("TLS_ECDHE_RSA_WITH_AES_128_GCM_SHA256");
+        // Use TLSv1.2 (as TLSv1.1 and older are now disabled in Java 8u292)
+        clientTLSFactory.setIncludeProtocols("TLSv1.2");
+        // Use only a cipher suite that exists for TLSv1.3 (this is the mismatch)
+        clientTLSFactory.setIncludeCipherSuites("TLS_AES_256_GCM_SHA384");
         startClient(clientTLSFactory);
 
+        // If this JVM has "TLSv1.3" present, then it's assumed that it also has the new logic
+        // to not send bytes to the server when a protocol and cipher suite mismatch occurs
+        // on the client side configuration.
+        List<String> supportedClientProtocols = Arrays.asList(clientTLSFactory.getSslContext().getSupportedSSLParameters().getProtocols());
+        boolean expectServerFailure = !(supportedClientProtocols.contains("TLSv1.3"));
+
         CountDownLatch clientLatch = new CountDownLatch(1);
         client.addBean(new SslHandshakeListener()
         {
@@ -301,7 +324,8 @@ public class HttpClientTLSTest
                 .timeout(5, TimeUnit.SECONDS)
                 .send());
 
-        assertTrue(serverLatch.await(1, TimeUnit.SECONDS));
+        if (expectServerFailure)
+            assertTrue(serverLatch.await(1, TimeUnit.SECONDS));
         assertTrue(clientLatch.await(1, TimeUnit.SECONDS));
     }
 
@@ -343,7 +367,7 @@ public class HttpClientTLSTest
 
     // Excluded in JDK 11+ because resumed sessions cannot be compared
     // using their session IDs even though they are resumed correctly.
-    @EnabledOnJre({JRE.JAVA_8, JRE.JAVA_9, JRE.JAVA_10})
+    @EnabledForJreRange(max = JRE.JAVA_10)
     @Test
     public void testHandshakeSucceededWithSessionResumption() throws Exception
     {
@@ -423,7 +447,7 @@ public class HttpClientTLSTest
 
     // Excluded in JDK 11+ because resumed sessions cannot be compared
     // using their session IDs even though they are resumed correctly.
-    @EnabledOnJre({JRE.JAVA_8, JRE.JAVA_9, JRE.JAVA_10})
+    @EnabledForJreRange(max = JRE.JAVA_10)
     @Test
     public void testClientRawCloseDoesNotInvalidateSession() throws Exception
     {
@@ -924,4 +948,149 @@ public class HttpClientTLSTest
         // The HTTP request will resume and be forced to handle the TLS buffer expansion.
         assertTrue(responseLatch.await(5, TimeUnit.SECONDS));
     }
+
+    @Test
+    public void testDefaultNonDomainSNI() throws Exception
+    {
+        SslContextFactory.Server serverTLS = new SslContextFactory.Server();
+        serverTLS.setKeyStorePath("src/test/resources/keystore_sni_non_domain.p12");
+        serverTLS.setKeyStorePassword("storepwd");
+        serverTLS.setSNISelector((keyType, issuers, session, sniHost, certificates) ->
+        {
+            // Java clients don't send SNI by default if it's not a domain.
+            assertNull(sniHost);
+            return serverTLS.sniSelect(keyType, issuers, session, sniHost, certificates);
+        });
+        startServer(serverTLS, new EmptyServerHandler());
+
+        SslContextFactory.Client clientTLS = new SslContextFactory.Client();
+        // Trust any certificate received by the server.
+        clientTLS.setTrustStorePath("src/test/resources/keystore_sni_non_domain.p12");
+        clientTLS.setTrustStorePassword("storepwd");
+        // Disable TLS-level hostName verification, as we may receive a random certificate.
+        clientTLS.setEndpointIdentificationAlgorithm(null);
+        startClient(clientTLS);
+
+        // Host is "localhost" which is not a domain, so the JDK won't send SNI.
+        ContentResponse response = client.newRequest("localhost", connector.getLocalPort())
+            .scheme(HttpScheme.HTTPS.asString())
+            .send();
+
+        assertEquals(HttpStatus.OK_200, response.getStatus());
+    }
+
+    @Test
+    public void testForcedNonDomainSNI() throws Exception
+    {
+        SslContextFactory.Server serverTLS = new SslContextFactory.Server();
+        serverTLS.setKeyStorePath("src/test/resources/keystore_sni_non_domain.p12");
+        serverTLS.setKeyStorePassword("storepwd");
+        serverTLS.setSNISelector((keyType, issuers, session, sniHost, certificates) ->
+        {
+            // We have forced the client to send the non-domain SNI.
+            assertNotNull(sniHost);
+            return serverTLS.sniSelect(keyType, issuers, session, sniHost, certificates);
+        });
+        startServer(serverTLS, new EmptyServerHandler());
+
+        SslContextFactory.Client clientTLS = new SslContextFactory.Client();
+        // Trust any certificate received by the server.
+        clientTLS.setTrustStorePath("src/test/resources/keystore_sni_non_domain.p12");
+        clientTLS.setTrustStorePassword("storepwd");
+        // Force TLS-level hostName verification, as we want to receive the correspondent certificate.
+        clientTLS.setEndpointIdentificationAlgorithm("HTTPS");
+        startClient(clientTLS);
+        clientTLS.setSNIProvider(SslContextFactory.Client.SniProvider.NON_DOMAIN_SNI_PROVIDER);
+
+        // Send a request with SNI "localhost", we should get the certificate at alias=localhost.
+        ContentResponse response1 = client.newRequest("localhost", connector.getLocalPort())
+            .scheme(HttpScheme.HTTPS.asString())
+            .send();
+        assertEquals(HttpStatus.OK_200, response1.getStatus());
+
+        // Send a request with SNI "127.0.0.1", we should get the certificate at alias=ip.
+        ContentResponse response2 = client.newRequest("127.0.0.1", connector.getLocalPort())
+            .scheme(HttpScheme.HTTPS.asString())
+            .send();
+        assertEquals(HttpStatus.OK_200, response2.getStatus());
+    }
+
+    @Test
+    @EnabledForJreRange(max = JRE.JAVA_16, disabledReason = "Since Java 17, SNI host names can only have letter|digit|hyphen characters.")
+    public void testForcedNonDomainSNIWithIPv6() throws Exception
+    {
+        Assumptions.assumeTrue(Net.isIpv6InterfaceAvailable());
+
+        SslContextFactory.Server serverTLS = new SslContextFactory.Server();
+        serverTLS.setKeyStorePath("src/test/resources/keystore_sni_non_domain.p12");
+        serverTLS.setKeyStorePassword("storepwd");
+        serverTLS.setSNISelector((keyType, issuers, session, sniHost, certificates) ->
+        {
+            // We have forced the client to send the non-domain SNI.
+            assertNotNull(sniHost);
+            return serverTLS.sniSelect(keyType, issuers, session, sniHost, certificates);
+        });
+        startServer(serverTLS, new EmptyServerHandler());
+
+        SslContextFactory.Client clientTLS = new SslContextFactory.Client();
+        // Trust any certificate received by the server.
+        clientTLS.setTrustStorePath("src/test/resources/keystore_sni_non_domain.p12");
+        clientTLS.setTrustStorePassword("storepwd");
+        // Force TLS-level hostName verification, as we want to receive the correspondent certificate.
+        clientTLS.setEndpointIdentificationAlgorithm("HTTPS");
+        startClient(clientTLS);
+        clientTLS.setSNIProvider(SslContextFactory.Client.SniProvider.NON_DOMAIN_SNI_PROVIDER);
+
+        // Send a request with SNI "[::1]", we should get the certificate at alias=ip.
+        ContentResponse response3 = client.newRequest("[::1]", connector.getLocalPort())
+            .scheme(HttpScheme.HTTPS.asString())
+            .send();
+
+        assertEquals(HttpStatus.OK_200, response3.getStatus());
+    }
+
+    @Test
+    public void testBytesInBytesOut() throws Exception
+    {
+        // Two connections will be closed: SslConnection and HttpConnection.
+        // Two on the server, two on the client.
+        CountDownLatch latch = new CountDownLatch(4);
+        SslContextFactory serverTLSFactory = createServerSslContextFactory();
+        startServer(serverTLSFactory, new EmptyServerHandler());
+        ConnectionStatistics serverStats = new ConnectionStatistics()
+        {
+            @Override
+            public void onClosed(Connection connection)
+            {
+                super.onClosed(connection);
+                latch.countDown();
+            }
+        };
+        connector.addManaged(serverStats);
+
+        SslContextFactory clientTLSFactory = createClientSslContextFactory();
+        startClient(clientTLSFactory);
+        ConnectionStatistics clientStats = new ConnectionStatistics()
+        {
+            @Override
+            public void onClosed(Connection connection)
+            {
+                super.onClosed(connection);
+                latch.countDown();
+            }
+        };
+        client.addManaged(clientStats);
+
+        ContentResponse response = client.newRequest("https://localhost:" + connector.getLocalPort())
+            .header(HttpHeader.CONNECTION, HttpHeaderValue.CLOSE.asString())
+            .send();
+        assertEquals(HttpStatus.OK_200, response.getStatus());
+
+        assertTrue(latch.await(5, TimeUnit.SECONDS));
+
+        assertThat(clientStats.getSentBytes(), Matchers.greaterThan(0L));
+        assertEquals(clientStats.getSentBytes(), serverStats.getReceivedBytes());
+        assertThat(clientStats.getReceivedBytes(), Matchers.greaterThan(0L));
+        assertEquals(clientStats.getReceivedBytes(), serverStats.getSentBytes());
+    }
 }
