@@ -1,6 +1,6 @@
 //
 //  ========================================================================
-//  Copyright (c) 1995-2021 Mort Bay Consulting Pty Ltd and others.
+//  Copyright (c) 1995-2022 Mort Bay Consulting Pty Ltd and others.
 //  ------------------------------------------------------------------------
 //  All rights reserved. This program and the accompanying materials
 //  are made available under the terms of the Eclipse Public License v1.0
@@ -36,22 +36,47 @@ import org.eclipse.jetty.util.UrlEncoded;
 /**
  * Http URI.
  * Parse an HTTP URI from a string or byte array.  Given a URI
- * <code>http://user@host:port/path/info;param?query#fragment</code>
- * this class will split it into the following undecoded optional elements:<ul>
+ * {@code http://user@host:port/path;param1/%2e/info;param2?query#fragment}
+ * this class will split it into the following optional elements:<ul>
  * <li>{@link #getScheme()} - http:</li>
  * <li>{@link #getAuthority()} - //name@host:port</li>
  * <li>{@link #getHost()} - host</li>
  * <li>{@link #getPort()} - port</li>
- * <li>{@link #getPath()} - /path/info</li>
- * <li>{@link #getParam()} - param</li>
+ * <li>{@link #getPath()} - /path;param1/%2e/info;param2</li>
+ * <li>{@link #getDecodedPath()} - /path/info</li>
+ * <li>{@link #getParam()} - param2</li>
  * <li>{@link #getQuery()} - query</li>
  * <li>{@link #getFragment()} - fragment</li>
  * </ul>
  *
- * <p>Any parameters will be returned from {@link #getPath()}, but are excluded from the
- * return value of {@link #getDecodedPath()}.   If there are multiple parameters, the
- * {@link #getParam()} method returns only the last one.
- */
+ * <p>The path part of the URI is provided in both raw form ({@link #getPath()}) and
+ * decoded form ({@link #getDecodedPath}), which has: path parameters removed,
+ * percent encoded characters expanded and relative segments resolved.  This approach
+ * is somewhat contrary to <a href="https://tools.ietf.org/html/rfc3986#section-3.3">RFC3986</a>
+ * which no longer defines path parameters (removed after
+ * <a href="https://tools.ietf.org/html/rfc2396#section-3.3">RFC2396</a>) and specifies
+ * that relative segment normalization should take place before percent encoded character
+ * expansion. A literal interpretation of the RFC can result in URI paths with ambiguities
+ * when viewed as strings. For example, a URI of {@code /foo%2f..%2fbar} is technically a single
+ * segment of "/foo/../bar", but could easily be misinterpreted as 3 segments resolving to "/bar"
+ * by a file system.
+ * </p>
+ * <p>
+ * Thus this class avoid and/or detects such ambiguities. Furthermore, by decoding characters and
+ * removing parameters before relative path normalization, ambiguous paths will be resolved in such
+ * a way to be non-standard-but-non-ambiguous to down stream interpretation of the decoded path string.
+ * The violations are recorded and available by API such as {@link #hasAmbiguousSegment()} so that requests
+ * containing them may be rejected in case the non-standard-but-non-ambiguous interpretations
+ * are not satisfactory for a given compliance configuration.
+ * </p>
+ * <p>
+ * Implementations that wish to process ambiguous URI paths must configure the compliance modes
+ * to accept them and then perform their own decoding of {@link #getPath()}.
+ * </p>
+ * <p>
+ * If there are multiple path parameters, only the last one is returned by {@link #getParam()}.
+ * </p>
+ **/
 public class HttpURI
 {
     private enum State
@@ -69,35 +94,65 @@ public class HttpURI
         ASTERISK
     }
 
-    enum Ambiguous
+    /**
+     * Violations of safe URI interpretations
+     */
+    enum Violation
     {
-        SEGMENT,
-        SEPARATOR,
-        PARAM
+        /**
+         * Ambiguous path segments e.g. {@code /foo/%2E%2E/bar}
+         */
+        SEGMENT("Ambiguous path segments"),
+        /**
+         * Ambiguous path separator within a URI segment e.g. {@code /foo%2Fbar}
+         */
+        SEPARATOR("Ambiguous path separator"),
+        /**
+         * Ambiguous path parameters within a URI segment e.g. {@code /foo/..;/bar} or {@code /foo/%2e%2e;param/bar}
+         */
+        PARAM("Ambiguous path parameters"),
+        /**
+         * Ambiguous double encoding within a URI segment e.g. {@code /%2557EB-INF}
+         */
+        ENCODING("Ambiguous double encoding"),
+        /**
+         * Ambiguous empty segments e.g. {@code /foo//bar}
+         */
+        EMPTY("Ambiguous empty segments"),
+        /**
+         * Non standard UTF-16 encoding eg {@code /foo%u2192bar}.
+         */
+        UTF16("Non standard UTF-16 encoding");
+
+        private final String _message;
+
+        Violation(String message)
+        {
+            _message = message;
+        }
+
+        String getMessage()
+        {
+            return _message;
+        }
     }
 
-    /**
-     * The concept of URI path parameters was originally specified in
-     * <a href="https://tools.ietf.org/html/rfc2396#section-3.3">RFC2396</a>, but that was
-     * obsoleted by
-     * <a href="https://tools.ietf.org/html/rfc3986#section-3.3">RFC3986</a> which removed
-     * a normative definition of path parameters. Specifically it excluded them from the
-     * <a href="https://tools.ietf.org/html/rfc3986#section-5.2.4">Remove Dot Segments</a>
-     * algorithm.  This results in some ambiguity as dot segments can result from later
-     * parameter removal or % encoding expansion, that are not removed from the URI
-     * by {@link URIUtil#canonicalPath(String)}.  Thus this class flags such ambiguous
-     * path segments, so that they may be rejected by the server if so configured.
-     */
     private static final Trie<Boolean> __ambiguousSegments = new ArrayTrie<>();
 
     static
     {
-        __ambiguousSegments.put("%2e", Boolean.TRUE);
-        __ambiguousSegments.put("%2e%2e", Boolean.TRUE);
-        __ambiguousSegments.put(".%2e", Boolean.TRUE);
-        __ambiguousSegments.put("%2e.", Boolean.TRUE);
-        __ambiguousSegments.put("..", Boolean.FALSE);
         __ambiguousSegments.put(".", Boolean.FALSE);
+        __ambiguousSegments.put("%2e", Boolean.TRUE);
+        __ambiguousSegments.put("%u002e", Boolean.TRUE);
+        __ambiguousSegments.put("..", Boolean.FALSE);
+        __ambiguousSegments.put(".%2e", Boolean.TRUE);
+        __ambiguousSegments.put(".%u002e", Boolean.TRUE);
+        __ambiguousSegments.put("%2e.", Boolean.TRUE);
+        __ambiguousSegments.put("%2e%2e", Boolean.TRUE);
+        __ambiguousSegments.put("%2e%u002e", Boolean.TRUE);
+        __ambiguousSegments.put("%u002e.", Boolean.TRUE);
+        __ambiguousSegments.put("%u002e%2e", Boolean.TRUE);
+        __ambiguousSegments.put("%u002e%u002e", Boolean.TRUE);
     }
 
     private String _scheme;
@@ -110,7 +165,8 @@ public class HttpURI
     private String _fragment;
     private String _uri;
     private String _decodedPath;
-    private final EnumSet<Ambiguous> _ambiguous = EnumSet.noneOf(Ambiguous.class);
+    private final EnumSet<Violation> _violations = EnumSet.noneOf(Violation.class);
+    private boolean _emptySegment;
 
     /**
      * Construct a normalized URI.
@@ -165,7 +221,24 @@ public class HttpURI
         _fragment = uri._fragment;
         _uri = uri._uri;
         _decodedPath = uri._decodedPath;
-        _ambiguous.addAll(uri._ambiguous);
+        _violations.addAll(uri._violations);
+        _emptySegment = false;
+    }
+
+    public HttpURI(HttpURI schemeHostPort, HttpURI uri)
+    {
+        _scheme = schemeHostPort._scheme;
+        _user = schemeHostPort._user;
+        _host = schemeHostPort._host;
+        _port = schemeHostPort._port;
+        _path = uri._path;
+        _param = uri._param;
+        _query = uri._query;
+        _fragment = uri._fragment;
+        _uri = uri._uri;
+        _decodedPath = uri._decodedPath;
+        _violations.addAll(uri._violations);
+        _emptySegment = false;
     }
 
     public HttpURI(String uri)
@@ -212,7 +285,8 @@ public class HttpURI
         _query = null;
         _fragment = null;
         _decodedPath = null;
-        _ambiguous.clear();
+        _emptySegment = false;
+        _violations.clear();
     }
 
     public void parse(String uri)
@@ -234,7 +308,7 @@ public class HttpURI
         _uri = uri;
 
         if (HttpMethod.CONNECT.is(method))
-            _path = uri;
+            parse(State.HOST, uri, 0, uri.length());
         else
             parse(uri.startsWith("/") ? State.PATH : State.START, uri, 0, uri.length());
     }
@@ -260,9 +334,11 @@ public class HttpURI
         int mark = offset; // the start of the current section being parsed
         int pathMark = 0; // the start of the path section
         int segment = 0; // the start of the current segment within the path
-        boolean encoded = false; // set to true if the path contains % encoded characters
-        boolean dot = false; // set to true if the path containers . or .. segments
-        int escapedSlash = 0; // state of parsing a %2f
+        boolean encodedPath = false; // set to true if the path contains % encoded characters
+        boolean encodedUtf16 = false; // Is the current encoding for UTF16?
+        int encodedCharacters = 0; // partial state of parsing a % encoded character<x>
+        int encodedValue = 0; // the partial encoded value
+        boolean dot = false; // set to true if the path contains . or .. segments
 
         for (int i = offset; i < end; i++)
         {
@@ -279,16 +355,21 @@ public class HttpURI
                             state = State.HOST_OR_PATH;
                             break;
                         case ';':
+                            checkSegment(uri, segment, i, true);
                             mark = i + 1;
                             state = State.PARAM;
                             break;
                         case '?':
                             // assume empty path (if seen at start)
+                            checkSegment(uri, segment, i, false);
                             _path = "";
                             mark = i + 1;
                             state = State.QUERY;
                             break;
                         case '#':
+                            // assume empty path (if seen at start)
+                            checkSegment(uri, segment, i, false);
+                            _path = "";
                             mark = i + 1;
                             state = State.FRAGMENT;
                             break;
@@ -297,12 +378,13 @@ public class HttpURI
                             state = State.ASTERISK;
                             break;
                         case '%':
-                            encoded = true;
-                            escapedSlash = 1;
+                            encodedPath = true;
+                            encodedCharacters = 2;
+                            encodedValue = 0;
                             mark = pathMark = segment = i;
                             state = State.PATH;
                             break;
-                        case '.' :
+                        case '.':
                             dot = true;
                             pathMark = segment = i;
                             state = State.PATH;
@@ -316,10 +398,11 @@ public class HttpURI
                                 pathMark = segment = i;
                                 state = State.PATH;
                             }
+                            break;
                     }
-
                     continue;
                 }
+
                 case SCHEME_OR_PATH:
                 {
                     switch (c)
@@ -336,24 +419,25 @@ public class HttpURI
                             state = State.PATH;
                             break;
                         case ';':
-                            // must have been in a path 
+                            // must have been in a path
                             mark = i + 1;
                             state = State.PARAM;
                             break;
                         case '?':
-                            // must have been in a path 
+                            // must have been in a path
                             _path = uri.substring(mark, i);
                             mark = i + 1;
                             state = State.QUERY;
                             break;
                         case '%':
-                            // must have be in an encoded path 
-                            encoded = true;
-                            escapedSlash = 1;
+                            // must have been in an encoded path
+                            encodedPath = true;
+                            encodedCharacters = 2;
+                            encodedValue = 0;
                             state = State.PATH;
                             break;
                         case '#':
-                            // must have been in a path 
+                            // must have been in a path
                             _path = uri.substring(mark, i);
                             state = State.FRAGMENT;
                             break;
@@ -371,7 +455,6 @@ public class HttpURI
                             mark = i + 1;
                             state = State.HOST;
                             break;
-
                         case '%':
                         case '@':
                         case ';':
@@ -392,6 +475,7 @@ public class HttpURI
                     }
                     continue;
                 }
+
                 case HOST:
                 {
                     switch (c)
@@ -420,7 +504,7 @@ public class HttpURI
                         default:
                             break;
                     }
-                    continue;
+                    break;
                 }
                 case IPV6:
                 {
@@ -445,7 +529,7 @@ public class HttpURI
                         default:
                             break;
                     }
-                    continue;
+                    break;
                 }
                 case PORT:
                 {
@@ -465,54 +549,81 @@ public class HttpURI
                         segment = i + 1;
                         state = State.PATH;
                     }
-                    continue;
+                    break;
                 }
                 case PATH:
                 {
-                    switch (c)
+                    if (encodedCharacters > 0)
                     {
-                        case ';':
-                            checkSegment(uri, segment, i, true);
-                            mark = i + 1;
-                            state = State.PARAM;
-                            break;
-                        case '?':
-                            checkSegment(uri, segment, i, false);
-                            _path = uri.substring(pathMark, i);
-                            mark = i + 1;
-                            state = State.QUERY;
-                            break;
-                        case '#':
-                            checkSegment(uri, segment, i, false);
-                            _path = uri.substring(pathMark, i);
-                            mark = i + 1;
-                            state = State.FRAGMENT;
-                            break;
-                        case '/':
-                            checkSegment(uri, segment, i, false);
-                            segment = i + 1;
-                            break;
-                        case '.':
-                            dot |= segment == i;
-                            break;
-                        case '%':
-                            encoded = true;
-                            escapedSlash = 1;
-                            break;
-                        case '2':
-                            escapedSlash = escapedSlash == 1 ? 2 : 0;
-                            break;
-                        case 'f':
-                        case 'F':
-                            if (escapedSlash == 2)
-                                _ambiguous.add(Ambiguous.SEPARATOR);
-                            escapedSlash = 0;
-                            break;
-                        default:
-                            escapedSlash = 0;
-                            break;
+                        if (encodedCharacters == 2 && c == 'u' && !encodedUtf16)
+                        {
+                            _violations.add(Violation.UTF16);
+                            encodedUtf16 = true;
+                            encodedCharacters = 4;
+                            continue;
+                        }
+                        encodedValue = (encodedValue << 4) + TypeUtil.convertHexDigit(c);
+
+                        if (--encodedCharacters == 0)
+                        {
+                            switch (encodedValue)
+                            {
+                                case 0:
+                                    // Byte 0 cannot be present in a UTF-8 sequence in any position
+                                    // other than as the NUL ASCII byte which we do not wish to allow.
+                                    throw new IllegalArgumentException("Illegal character in path");
+                                case '/':
+                                    _violations.add(Violation.SEPARATOR);
+                                    break;
+                                case '%':
+                                    _violations.add(Violation.ENCODING);
+                                    break;
+                                default:
+                                    break;
+                            }
+                        }
                     }
-                    continue;
+                    else
+                    {
+                        switch (c)
+                        {
+                            case ';':
+                                checkSegment(uri, segment, i, true);
+                                mark = i + 1;
+                                state = State.PARAM;
+                                break;
+                            case '?':
+                                checkSegment(uri, segment, i, false);
+                                _path = uri.substring(pathMark, i);
+                                mark = i + 1;
+                                state = State.QUERY;
+                                break;
+                            case '#':
+                                checkSegment(uri, segment, i, false);
+                                _path = uri.substring(pathMark, i);
+                                mark = i + 1;
+                                state = State.FRAGMENT;
+                                break;
+                            case '/':
+                                // There is no leading segment when parsing only a path that starts with slash.
+                                if (i != 0)
+                                    checkSegment(uri, segment, i, false);
+                                segment = i + 1;
+                                break;
+                            case '.':
+                                dot |= segment == i;
+                                break;
+                            case '%':
+                                encodedPath = true;
+                                encodedUtf16 = false;
+                                encodedCharacters = 2;
+                                encodedValue = 0;
+                                break;
+                            default:
+                                break;
+                        }
+                    }
+                    break;
                 }
                 case PARAM:
                 {
@@ -531,7 +642,7 @@ public class HttpURI
                             state = State.FRAGMENT;
                             break;
                         case '/':
-                            encoded = true;
+                            encodedPath = true;
                             segment = i + 1;
                             state = State.PATH;
                             break;
@@ -542,7 +653,7 @@ public class HttpURI
                         default:
                             break;
                     }
-                    continue;
+                    break;
                 }
                 case QUERY:
                 {
@@ -552,7 +663,7 @@ public class HttpURI
                         mark = i + 1;
                         state = State.FRAGMENT;
                     }
-                    continue;
+                    break;
                 }
                 case ASTERISK:
                 {
@@ -565,17 +676,21 @@ public class HttpURI
                     break;
                 }
                 default:
-                    break;
+                {
+                    throw new IllegalStateException(state.toString());
+                }
             }
         }
 
         switch (state)
         {
             case START:
+                _path = "";
+                checkSegment(uri, segment, end, false);
+                break;
+            case ASTERISK:
                 break;
             case SCHEME_OR_PATH:
-                _path = uri.substring(mark, end);
-                break;
             case HOST_OR_PATH:
                 _path = uri.substring(mark, end);
                 break;
@@ -588,11 +703,6 @@ public class HttpURI
             case PORT:
                 _port = TypeUtil.parseInt(uri, mark, end - mark, 10);
                 break;
-            case ASTERISK:
-                break;
-            case FRAGMENT:
-                _fragment = uri.substring(mark, end);
-                break;
             case PARAM:
                 _path = uri.substring(pathMark, end);
                 _param = uri.substring(mark, end);
@@ -604,11 +714,14 @@ public class HttpURI
             case QUERY:
                 _query = uri.substring(mark, end);
                 break;
-            default:
+            case FRAGMENT:
+                _fragment = uri.substring(mark, end);
                 break;
+            default:
+                throw new IllegalStateException(state.toString());
         }
 
-        if (!encoded && !dot)
+        if (!encodedPath && !dot)
         {
             if (_param == null)
                 _decodedPath = _path;
@@ -617,10 +730,12 @@ public class HttpURI
         }
         else if (_path != null)
         {
-            String canonical = URIUtil.canonicalPath(_path);
-            if (canonical == null)
-                throw new BadMessageException("Bad URI");
-            _decodedPath = URIUtil.decodePath(canonical);
+            // The RFC requires this to be canonical before decoding, but this can leave dot segments and dot dot segments
+            // which are not canonicalized and could be used in an attempt to bypass security checks.
+            String decodedNonCanonical = URIUtil.decodePath(_path);
+            _decodedPath = URIUtil.canonicalPath(decodedNonCanonical);
+            if (_decodedPath == null)
+                throw new IllegalArgumentException("Bad URI");
         }
     }
 
@@ -629,19 +744,52 @@ public class HttpURI
      *
      * An ambiguous path segment is one that is perhaps technically legal, but is considered undesirable to handle
      * due to possible ambiguity.  Examples include segments like '..;', '%2e', '%2e%2e' etc.
+     *
      * @param uri The URI string
      * @param segment The inclusive starting index of the segment (excluding any '/')
      * @param end The exclusive end index of the segment
      */
     private void checkSegment(String uri, int segment, int end, boolean param)
     {
-        if (!_ambiguous.contains(Ambiguous.SEGMENT))
+        // This method is called once for every segment parsed.
+        // A URI like "/foo/" has two segments: "foo" and an empty segment.
+        // Empty segments are only ambiguous if they are not the last segment
+        // So if this method is called for any segment and we have previously
+        // seen an empty segment, then it was ambiguous.
+        if (_emptySegment)
+            _violations.add(Violation.EMPTY);
+
+        if (end == segment)
         {
-            Boolean ambiguous = __ambiguousSegments.get(uri, segment, end - segment);
-            if (ambiguous == Boolean.TRUE)
-                _ambiguous.add(Ambiguous.SEGMENT);
-            else if (param && ambiguous == Boolean.FALSE)
-                _ambiguous.add(Ambiguous.PARAM);
+            // Empty segments are not ambiguous if followed by a '#', '?' or end of string.
+            if (end >= uri.length() || ("#?".indexOf(uri.charAt(end)) >= 0))
+                return;
+
+            // If this empty segment is the first segment then it is ambiguous.
+            if (segment == 0)
+            {
+                _violations.add(Violation.EMPTY);
+                return;
+            }
+
+            // Otherwise remember we have seen an empty segment, which is check if we see a subsequent segment.
+            if (!_emptySegment)
+            {
+                _emptySegment = true;
+                return;
+            }
+        }
+
+        // Look for segment in the ambiguous segment index.
+        Boolean ambiguous = __ambiguousSegments.get(uri, segment, end - segment);
+        if (ambiguous != null)
+        {
+            // Is the segment intrinsically ambiguous
+            if (Boolean.TRUE.equals(ambiguous))
+                _violations.add(Violation.SEGMENT);
+            // Is the segment ambiguous with a parameter?
+            if (param)
+                _violations.add(Violation.PARAM);
         }
     }
 
@@ -650,7 +798,15 @@ public class HttpURI
      */
     public boolean hasAmbiguousSegment()
     {
-        return _ambiguous.contains(Ambiguous.SEGMENT);
+        return _violations.contains(Violation.SEGMENT);
+    }
+
+    /**
+     * @return True if the URI empty segment that is ambiguous like '//' or '/;param/'.
+     */
+    public boolean hasAmbiguousEmptySegment()
+    {
+        return _violations.contains(Violation.EMPTY);
     }
 
     /**
@@ -658,7 +814,7 @@ public class HttpURI
      */
     public boolean hasAmbiguousSeparator()
     {
-        return _ambiguous.contains(Ambiguous.SEPARATOR);
+        return _violations.contains(Violation.SEPARATOR);
     }
 
     /**
@@ -666,15 +822,45 @@ public class HttpURI
      */
     public boolean hasAmbiguousParameter()
     {
-        return _ambiguous.contains(Ambiguous.PARAM);
+        return _violations.contains(Violation.PARAM);
     }
 
     /**
-     * @return True if the URI has either an {@link #hasAmbiguousSegment()} or {@link #hasAmbiguousSeparator()}.
+     * @return True if the URI has an encoded '%' character.
+     */
+    public boolean hasAmbiguousEncoding()
+    {
+        return _violations.contains(Violation.ENCODING);
+    }
+
+    /**
+     * @return True if the URI has either an {@link #hasAmbiguousSegment()} or {@link #hasAmbiguousEmptySegment()}
+     * or {@link #hasAmbiguousSeparator()} or {@link #hasAmbiguousParameter()}
      */
     public boolean isAmbiguous()
     {
-        return !_ambiguous.isEmpty();
+        return !_violations.isEmpty() && !(_violations.size() == 1 && _violations.contains(Violation.UTF16));
+    }
+
+    /**
+     * @return True if the URI has any Violations.
+     */
+    public boolean hasViolations()
+    {
+        return !_violations.isEmpty();
+    }
+
+    boolean hasViolation(Violation violation)
+    {
+        return _violations.contains(violation);
+    }
+
+    /**
+     * @return True if the URI encodes UTF-16 characters with '%u'.
+     */
+    public boolean hasUtf16Encoding()
+    {
+        return _violations.contains(Violation.UTF16);
     }
 
     public String getScheme()
@@ -714,6 +900,11 @@ public class HttpURI
         return _decodedPath;
     }
 
+    /**
+     * Get a URI path parameter. Multiple and in segment parameters are ignored and only
+     * the last trailing parameter is returned.
+     * @return The last path parameter or null
+     */
     public String getParam()
     {
         return _param;
@@ -841,16 +1032,29 @@ public class HttpURI
      */
     public void setAuthority(String host, int port)
     {
+        if (host != null && !isPathValidForAuthority(_path))
+            throw new IllegalArgumentException("Relative path with authority");
         _host = host;
         _port = port;
         _uri = null;
     }
 
+    private boolean isPathValidForAuthority(String path)
+    {
+        if (path == null)
+            return true;
+        if (path.isEmpty() || "*".equals(path))
+            return true;
+        return path.startsWith("/");
+    }
+
     /**
      * @param path the path
      */
     public void setPath(String path)
     {
+        if (hasAuthority() && !isPathValidForAuthority(path))
+            throw new IllegalArgumentException("Relative path with authority");
         _uri = null;
         _path = null;
         if (path != null)
@@ -859,6 +1063,8 @@ public class HttpURI
 
     public void setPathQuery(String pathQuery)
     {
+        if (hasAuthority() && !isPathValidForAuthority(pathQuery))
+            throw new IllegalArgumentException("Relative path with authority");
         _uri = null;
         _path = null;
         _decodedPath = null;
@@ -872,6 +1078,11 @@ public class HttpURI
             parse(State.PATH, pathQuery, 0, pathQuery.length());
     }
 
+    private boolean hasAuthority()
+    {
+        return _host != null;
+    }
+
     public void setQuery(String query)
     {
         _query = query;
