@@ -1,6 +1,6 @@
 //
 //  ========================================================================
-//  Copyright (c) 1995-2020 Mort Bay Consulting Pty Ltd and others.
+//  Copyright (c) 1995-2022 Mort Bay Consulting Pty Ltd and others.
 //  ------------------------------------------------------------------------
 //  All rights reserved. This program and the accompanying materials
 //  are made available under the terms of the Eclipse Public License v1.0
@@ -18,9 +18,8 @@
 
 package org.eclipse.jetty.http2.client;
 
-import java.util.concurrent.atomic.AtomicLong;
-
 import org.eclipse.jetty.http.MetaData;
+import org.eclipse.jetty.http2.CloseState;
 import org.eclipse.jetty.http2.ErrorCode;
 import org.eclipse.jetty.http2.FlowControlStrategy;
 import org.eclipse.jetty.http2.HTTP2Session;
@@ -29,7 +28,6 @@ import org.eclipse.jetty.http2.api.Session;
 import org.eclipse.jetty.http2.api.Stream;
 import org.eclipse.jetty.http2.frames.HeadersFrame;
 import org.eclipse.jetty.http2.frames.PushPromiseFrame;
-import org.eclipse.jetty.http2.frames.ResetFrame;
 import org.eclipse.jetty.http2.generator.Generator;
 import org.eclipse.jetty.io.EndPoint;
 import org.eclipse.jetty.util.Callback;
@@ -41,38 +39,11 @@ public class HTTP2ClientSession extends HTTP2Session
 {
     private static final Logger LOG = Log.getLogger(HTTP2ClientSession.class);
 
-    private final AtomicLong streamsOpened = new AtomicLong();
-    private final AtomicLong streamsClosed = new AtomicLong();
-
     public HTTP2ClientSession(Scheduler scheduler, EndPoint endPoint, Generator generator, Session.Listener listener, FlowControlStrategy flowControl)
     {
         super(scheduler, endPoint, generator, listener, flowControl, 1);
     }
 
-    @Override
-    protected void onStreamOpened(IStream stream)
-    {
-        super.onStreamOpened(stream);
-        streamsOpened.incrementAndGet();
-    }
-
-    @Override
-    protected void onStreamClosed(IStream stream)
-    {
-        super.onStreamClosed(stream);
-        streamsClosed.incrementAndGet();
-    }
-
-    public long getStreamsOpened()
-    {
-        return streamsOpened.get();
-    }
-
-    public long getStreamsClosed()
-    {
-        return streamsClosed.get();
-    }
-
     @Override
     public void onHeaders(HeadersFrame frame)
     {
@@ -92,6 +63,8 @@ public class HTTP2ClientSession extends HTTP2Session
             else
             {
                 stream.process(frame, Callback.NOOP);
+                if (stream.updateClose(frame.isEndStream(), CloseState.Event.RECEIVED))
+                    removeStream(stream);
                 notifyHeaders(stream, frame);
             }
         }
@@ -118,17 +91,6 @@ public class HTTP2ClientSession extends HTTP2Session
         }
     }
 
-    @Override
-    protected void onResetForUnknownStream(ResetFrame frame)
-    {
-        int streamId = frame.getStreamId();
-        boolean closed = isClientStream(streamId) ? isLocalStreamClosed(streamId) : isRemoteStreamClosed(streamId);
-        if (closed)
-            notifyReset(this, frame);
-        else
-            onConnectionFailure(ErrorCode.PROTOCOL_ERROR.code, "unexpected_rst_stream_frame");
-    }
-
     @Override
     public void onPushPromise(PushPromiseFrame frame)
     {
