@@ -1,6 +1,6 @@
 //
 //  ========================================================================
-//  Copyright (c) 1995-2020 Mort Bay Consulting Pty Ltd and others.
+//  Copyright (c) 1995-2022 Mort Bay Consulting Pty Ltd and others.
 //  ------------------------------------------------------------------------
 //  All rights reserved. This program and the accompanying materials
 //  are made available under the terms of the Eclipse Public License v1.0
@@ -29,6 +29,7 @@ import java.nio.charset.CharsetEncoder;
 import java.nio.charset.CoderResult;
 import java.nio.charset.CodingErrorAction;
 import java.util.ResourceBundle;
+import java.util.concurrent.CancellationException;
 import java.util.concurrent.TimeUnit;
 import javax.servlet.RequestDispatcher;
 import javax.servlet.ServletOutputStream;
@@ -37,6 +38,7 @@ import javax.servlet.ServletResponse;
 import javax.servlet.WriteListener;
 
 import org.eclipse.jetty.http.HttpContent;
+import org.eclipse.jetty.io.ByteBufferPool;
 import org.eclipse.jetty.io.EofException;
 import org.eclipse.jetty.util.BufferUtil;
 import org.eclipse.jetty.util.Callback;
@@ -68,7 +70,7 @@ public class HttpOutput extends ServletOutputStream implements Runnable
     enum State
     {
         OPEN,     // Open
-        CLOSE,    // Close needed from onWriteCompletion
+        CLOSE,    // Close needed from onWriteComplete
         CLOSING,  // Close in progress after close API called
         CLOSED    // Closed
     }
@@ -300,14 +302,14 @@ public class HttpOutput extends ServletOutputStream implements Runnable
                 _state = State.CLOSED;
                 closedCallback = _closedCallback;
                 _closedCallback = null;
-                releaseBuffer();
+                releaseBuffer(failure);
                 wake = updateApiState(failure);
             }
             else if (_state == State.CLOSE)
             {
                 // Somebody called close or complete while we were writing.
                 // We can now send a (probably empty) last buffer and then when it completes
-                // onWriteCompletion will be called again to actually execute the _completeCallback
+                // onWriteComplete will be called again to actually execute the _completeCallback
                 _state = State.CLOSING;
                 closeContent = BufferUtil.hasContent(_aggregate) ? _aggregate : BufferUtil.EMPTY_BUFFER;
             }
@@ -410,53 +412,87 @@ public class HttpOutput extends ServletOutputStream implements Runnable
         ByteBuffer content = null;
         synchronized (_channelState)
         {
-            switch (_state)
+            // First check the API state for any unrecoverable situations
+            switch (_apiState)
             {
-                case CLOSED:
-                    succeeded = true;
+                case UNREADY: // isReady() has returned false so a call to onWritePossible may happen at any time
+                    error = new CancellationException("Completed whilst write unready");
                     break;
 
-                case CLOSE:
-                case CLOSING:
-                    _closedCallback = Callback.combine(_closedCallback, callback);
+                case PENDING: // an async write is pending and may complete at any time
+                    // If this is not the last write, then we must abort
+                    if (!_channel.getResponse().isContentComplete(_written))
+                        error = new CancellationException("Completed whilst write pending");
                     break;
 
-                case OPEN:
-                    if (_onError != null)
-                    {
-                        error = _onError;
+                case BLOCKED: // another thread is blocked in a write or a close
+                    error = new CancellationException("Completed whilst write blocked");
+                    break;
+
+                default:
+                    break;
+            }
+
+            // If we can't complete due to the API state, then abort
+            if (error != null)
+            {
+                _channel.abort(error);
+                _writeBlocker.fail(error);
+                _state = State.CLOSED;
+            }
+            else
+            {
+                // Otherwise check the output state to determine how to complete
+                switch (_state)
+                {
+                    case CLOSED:
+                        succeeded = true;
                         break;
-                    }
 
-                    _closedCallback = Callback.combine(_closedCallback, callback);
+                    case CLOSE:
+                    case CLOSING:
+                        _closedCallback = Callback.combine(_closedCallback, callback);
+                        break;
 
-                    switch (_apiState)
-                    {
-                        case BLOCKING:
-                            // Output is idle blocking state, but we still do an async close
-                            _apiState = ApiState.BLOCKED;
-                            _state = State.CLOSING;
-                            content = BufferUtil.hasContent(_aggregate) ? _aggregate : BufferUtil.EMPTY_BUFFER;
+                    case OPEN:
+                        if (_onError != null)
+                        {
+                            error = _onError;
                             break;
+                        }
 
-                        case ASYNC:
-                        case READY:
-                            // Output is idle in async state, so we can do an async close
-                            _apiState = ApiState.PENDING;
-                            _state = State.CLOSING;
-                            content = BufferUtil.hasContent(_aggregate) ? _aggregate : BufferUtil.EMPTY_BUFFER;
-                            break;
+                        _closedCallback = Callback.combine(_closedCallback, callback);
 
-                        case BLOCKED:
-                        case UNREADY:
-                        case PENDING:
-                            // An operation is in progress, so we soft close now
-                            _softClose = true;
-                            // then trigger a close from onWriteComplete
-                            _state = State.CLOSE;
-                            break;
-                    }
-                    break;
+                        switch (_apiState)
+                        {
+                            case BLOCKING:
+                                // Output is idle blocking state, but we still do an async close
+                                _apiState = ApiState.BLOCKED;
+                                _state = State.CLOSING;
+                                content = BufferUtil.hasContent(_aggregate) ? _aggregate : BufferUtil.EMPTY_BUFFER;
+                                break;
+
+                            case ASYNC:
+                            case READY:
+                                // Output is idle in async state, so we can do an async close
+                                _apiState = ApiState.PENDING;
+                                _state = State.CLOSING;
+                                content = BufferUtil.hasContent(_aggregate) ? _aggregate : BufferUtil.EMPTY_BUFFER;
+                                break;
+
+                            case UNREADY:
+                            case PENDING:
+                                // An operation is in progress, so we soft close now
+                                _softClose = true;
+                                // then trigger a close from onWriteComplete
+                                _state = State.CLOSE;
+                                break;
+
+                            default:
+                                throw new IllegalStateException();
+                        }
+                        break;
+                }
             }
         }
 
@@ -482,13 +518,13 @@ public class HttpOutput extends ServletOutputStream implements Runnable
     /**
      * Called to indicate that the request cycle has been completed.
      */
-    public void completed()
+    public void completed(Throwable failure)
     {
         synchronized (_channelState)
         {
             _state = State.CLOSED;
+            releaseBuffer(failure);
         }
-        releaseBuffer();
     }
 
     @Override
@@ -606,6 +642,7 @@ public class HttpOutput extends ServletOutputStream implements Runnable
                 catch (Throwable t)
                 {
                     onWriteComplete(true, t);
+                    throw t;
                 }
             }
         }
@@ -613,21 +650,28 @@ public class HttpOutput extends ServletOutputStream implements Runnable
 
     public ByteBuffer getBuffer()
     {
-        return _aggregate;
+        synchronized (_channelState)
+        {
+            return acquireBuffer();
+        }
     }
 
-    public ByteBuffer acquireBuffer()
+    private ByteBuffer acquireBuffer()
     {
         if (_aggregate == null)
             _aggregate = _channel.getByteBufferPool().acquire(getBufferSize(), _interceptor.isOptimizedForDirectBuffers());
         return _aggregate;
     }
 
-    private void releaseBuffer()
+    private void releaseBuffer(Throwable failure)
     {
         if (_aggregate != null)
         {
-            _channel.getConnector().getByteBufferPool().release(_aggregate);
+            ByteBufferPool bufferPool = _channel.getConnector().getByteBufferPool();
+            if (failure == null)
+                bufferPool.release(_aggregate);
+            else
+                bufferPool.remove(_aggregate);
             _aggregate = null;
         }
     }
@@ -697,7 +741,9 @@ public class HttpOutput extends ServletOutputStream implements Runnable
         }
 
         if (content == null)
+        {
             new AsyncFlush(false).iterate();
+        }
         else
         {
             try
@@ -819,10 +865,12 @@ public class HttpOutput extends ServletOutputStream implements Runnable
         // Blocking write
         try
         {
+            boolean complete = false;
             // flush any content from the aggregate
             if (BufferUtil.hasContent(_aggregate))
             {
-                channelWrite(_aggregate, last && len == 0);
+                complete = last && len == 0;
+                channelWrite(_aggregate, complete);
 
                 // should we fill aggregate again from the buffer?
                 if (len > 0 && !last && len <= _commitSize && len <= maximizeAggregateSpace())
@@ -852,7 +900,7 @@ public class HttpOutput extends ServletOutputStream implements Runnable
                 }
                 channelWrite(view, last);
             }
-            else if (last)
+            else if (last && !complete)
             {
                 channelWrite(BufferUtil.EMPTY_BUFFER, true);
             }
@@ -879,7 +927,7 @@ public class HttpOutput extends ServletOutputStream implements Runnable
         {
             checkWritable();
             long written = _written + len;
-            last = _channel.getResponse().isAllContentWritten(_written);
+            last = _channel.getResponse().isAllContentWritten(written);
             flush = last || len > 0 || BufferUtil.hasContent(_aggregate);
 
             if (last && _state == State.OPEN)
@@ -923,13 +971,17 @@ public class HttpOutput extends ServletOutputStream implements Runnable
             {
                 // Blocking write
                 // flush any content from the aggregate
+                boolean complete = false;
                 if (BufferUtil.hasContent(_aggregate))
-                    channelWrite(_aggregate, last && len == 0);
+                {
+                    complete = last && len == 0;
+                    channelWrite(_aggregate, complete);
+                }
 
                 // write any remaining content in the buffer directly
                 if (len > 0)
                     channelWrite(buffer, last);
-                else if (last)
+                else if (last && !complete)
                     channelWrite(BufferUtil.EMPTY_BUFFER, true);
 
                 onWriteComplete(last, null);
@@ -1391,14 +1443,14 @@ public class HttpOutput extends ServletOutputStream implements Runnable
         {
             _state = State.OPEN;
             _apiState = ApiState.BLOCKING;
-            _softClose = false;
+            _softClose = true; // Stay closed until next request
             _interceptor = _channel;
             HttpConfiguration config = _channel.getHttpConfiguration();
             _bufferSize = config.getOutputBufferSize();
             _commitSize = config.getOutputAggregationSize();
             if (_commitSize > _bufferSize)
                 _commitSize = _bufferSize;
-            releaseBuffer();
+            releaseBuffer(null);
             _written = 0;
             _writeListener = null;
             _onError = null;
@@ -1410,10 +1462,13 @@ public class HttpOutput extends ServletOutputStream implements Runnable
 
     public void resetBuffer()
     {
-        _interceptor.resetBuffer();
-        if (BufferUtil.hasContent(_aggregate))
-            BufferUtil.clear(_aggregate);
-        _written = 0;
+        synchronized (_channelState)
+        {
+            _interceptor.resetBuffer();
+            if (BufferUtil.hasContent(_aggregate))
+                BufferUtil.clear(_aggregate);
+            _written = 0;
+        }
     }
 
     @Override
@@ -1526,7 +1581,7 @@ public class HttpOutput extends ServletOutputStream implements Runnable
     {
         final boolean _last;
 
-        ChannelWriteCB(boolean last)
+        private ChannelWriteCB(boolean last)
         {
             _last = last;
         }
@@ -1552,14 +1607,20 @@ public class HttpOutput extends ServletOutputStream implements Runnable
 
     private abstract class NestedChannelWriteCB extends ChannelWriteCB
     {
-        final Callback _callback;
+        private final Callback _callback;
 
-        NestedChannelWriteCB(Callback callback, boolean last)
+        private NestedChannelWriteCB(Callback callback, boolean last)
         {
             super(last);
             _callback = callback;
         }
 
+        @Override
+        public InvocationType getInvocationType()
+        {
+            return _callback.getInvocationType();
+        }
+
         @Override
         protected void onCompleteSuccess()
         {
@@ -1594,9 +1655,9 @@ public class HttpOutput extends ServletOutputStream implements Runnable
 
     private class AsyncFlush extends ChannelWriteCB
     {
-        volatile boolean _flushed;
+        private volatile boolean _flushed;
 
-        AsyncFlush(boolean last)
+        private AsyncFlush(boolean last)
         {
             super(last);
         }
@@ -1629,7 +1690,7 @@ public class HttpOutput extends ServletOutputStream implements Runnable
         private final int _len;
         private boolean _completed;
 
-        AsyncWrite(byte[] b, int off, int len, boolean last)
+        private AsyncWrite(byte[] b, int off, int len, boolean last)
         {
             super(last);
             _buffer = ByteBuffer.wrap(b, off, len);
@@ -1638,7 +1699,7 @@ public class HttpOutput extends ServletOutputStream implements Runnable
             _slice = _len < getBufferSize() ? null : _buffer.duplicate();
         }
 
-        AsyncWrite(ByteBuffer buffer, boolean last)
+        private AsyncWrite(ByteBuffer buffer, boolean last)
         {
             super(last);
             _buffer = buffer;
@@ -1726,7 +1787,7 @@ public class HttpOutput extends ServletOutputStream implements Runnable
         private boolean _eof;
         private boolean _closed;
 
-        InputStreamWritingCB(InputStream in, Callback callback)
+        private InputStreamWritingCB(InputStream in, Callback callback)
         {
             super(callback, true);
             _in = in;
@@ -1801,7 +1862,7 @@ public class HttpOutput extends ServletOutputStream implements Runnable
         private boolean _eof;
         private boolean _closed;
 
-        ReadableByteChannelWritingCB(ReadableByteChannel in, Callback callback)
+        private ReadableByteChannelWritingCB(ReadableByteChannel in, Callback callback)
         {
             super(callback, true);
             _in = in;
@@ -1882,5 +1943,11 @@ public class HttpOutput extends ServletOutputStream implements Runnable
         {
             onWriteComplete(true, x);
         }
+
+        @Override
+        public InvocationType getInvocationType()
+        {
+            return InvocationType.NON_BLOCKING;
+        }
     }
 }
