@@ -1,6 +1,6 @@
 //
 //  ========================================================================
-//  Copyright (c) 1995-2020 Mort Bay Consulting Pty Ltd and others.
+//  Copyright (c) 1995-2022 Mort Bay Consulting Pty Ltd and others.
 //  ------------------------------------------------------------------------
 //  All rights reserved. This program and the accompanying materials
 //  are made available under the terms of the Eclipse Public License v1.0
@@ -18,11 +18,12 @@
 
 package org.eclipse.jetty.server;
 
-import java.util.Arrays;
-import java.util.Iterator;
+import java.util.ArrayList;
 import java.util.List;
+import java.util.stream.Collectors;
 import javax.net.ssl.SSLEngine;
 
+import org.eclipse.jetty.http.HttpVersion;
 import org.eclipse.jetty.io.AbstractConnection;
 import org.eclipse.jetty.io.Connection;
 import org.eclipse.jetty.io.EndPoint;
@@ -30,13 +31,23 @@ import org.eclipse.jetty.io.ssl.SslConnection;
 
 public abstract class NegotiatingServerConnectionFactory extends AbstractConnectionFactory
 {
-    private final List<String> protocols;
+    private final List<String> negotiatedProtocols;
     private String defaultProtocol;
 
-    public NegotiatingServerConnectionFactory(String protocol, String... protocols)
+    public NegotiatingServerConnectionFactory(String protocol, String... negotiatedProtocols)
     {
         super(protocol);
-        this.protocols = Arrays.asList(protocols);
+        this.negotiatedProtocols = new ArrayList<>();
+        if (negotiatedProtocols != null)
+        {
+            // Trim the values, as they may come from XML configuration.
+            for (String p : negotiatedProtocols)
+            {
+                p = p.trim();
+                if (!p.isEmpty())
+                    this.negotiatedProtocols.add(p);
+            }
+        }
     }
 
     public String getDefaultProtocol()
@@ -46,36 +57,42 @@ public abstract class NegotiatingServerConnectionFactory extends AbstractConnect
 
     public void setDefaultProtocol(String defaultProtocol)
     {
-        this.defaultProtocol = defaultProtocol;
+        // Trim the value, as it may come from XML configuration.
+        String dft = defaultProtocol == null ? "" : defaultProtocol.trim();
+        this.defaultProtocol = dft.isEmpty() ? null : dft;
     }
 
-    public List<String> getProtocols()
+    public List<String> getNegotiatedProtocols()
     {
-        return protocols;
+        return negotiatedProtocols;
     }
 
     @Override
     public Connection newConnection(Connector connector, EndPoint endPoint)
     {
-        List<String> protocols = this.protocols;
-        if (protocols.isEmpty())
+        List<String> negotiated = this.negotiatedProtocols;
+        if (negotiated.isEmpty())
         {
-            protocols = connector.getProtocols();
-            Iterator<String> i = protocols.iterator();
-            while (i.hasNext())
-            {
-                String protocol = i.next();
-                String prefix = "ssl-";
-                if (protocol.regionMatches(true, 0, prefix, 0, prefix.length()) || protocol.equalsIgnoreCase("alpn"))
+            // Generate list of protocols that we can negotiate.
+            negotiated = connector.getProtocols().stream()
+                .filter(p ->
                 {
-                    i.remove();
-                }
-            }
+                    ConnectionFactory f = connector.getConnectionFactory(p);
+                    return !(f instanceof SslConnectionFactory) && !(f instanceof NegotiatingServerConnectionFactory);
+                })
+                .collect(Collectors.toList());
         }
 
+        // If default protocol is not set, then it is
+        // either HTTP/1.1 or the first protocol given.
         String dft = defaultProtocol;
-        if (dft == null && !protocols.isEmpty())
-            dft = protocols.get(0);
+        if (dft == null && !negotiated.isEmpty())
+        {
+            dft = negotiated.stream()
+                .filter(HttpVersion.HTTP_1_1::is)
+                .findFirst()
+                .orElse(negotiated.get(0));
+        }
 
         SSLEngine engine = null;
         EndPoint ep = endPoint;
@@ -88,7 +105,7 @@ public abstract class NegotiatingServerConnectionFactory extends AbstractConnect
                 ep = null;
         }
 
-        return configure(newServerConnection(connector, endPoint, engine, protocols, dft), connector, endPoint);
+        return configure(newServerConnection(connector, endPoint, engine, negotiated, dft), connector, endPoint);
     }
 
     protected abstract AbstractConnection newServerConnection(Connector connector, EndPoint endPoint, SSLEngine engine, List<String> protocols, String defaultProtocol);
@@ -96,6 +113,6 @@ public abstract class NegotiatingServerConnectionFactory extends AbstractConnect
     @Override
     public String toString()
     {
-        return String.format("%s@%x{%s,%s,%s}", getClass().getSimpleName(), hashCode(), getProtocol(), getDefaultProtocol(), getProtocols());
+        return String.format("%s@%x{%s,%s,%s}", getClass().getSimpleName(), hashCode(), getProtocols(), getDefaultProtocol(), getNegotiatedProtocols());
     }
 }
