@@ -1,6 +1,6 @@
 //
 //  ========================================================================
-//  Copyright (c) 1995-2020 Mort Bay Consulting Pty Ltd and others.
+//  Copyright (c) 1995-2022 Mort Bay Consulting Pty Ltd and others.
 //  ------------------------------------------------------------------------
 //  All rights reserved. This program and the accompanying materials
 //  are made available under the terms of the Eclipse Public License v1.0
@@ -19,6 +19,8 @@
 package org.eclipse.jetty.server;
 
 import java.security.cert.X509Certificate;
+import java.util.HashSet;
+import java.util.Set;
 import java.util.concurrent.TimeUnit;
 import javax.net.ssl.SSLContext;
 import javax.net.ssl.SSLEngine;
@@ -33,6 +35,8 @@ import org.eclipse.jetty.http.PreEncodedHttpField;
 import org.eclipse.jetty.io.EndPoint;
 import org.eclipse.jetty.io.ssl.SslConnection;
 import org.eclipse.jetty.io.ssl.SslConnection.DecryptedEndPoint;
+import org.eclipse.jetty.util.Attributes;
+import org.eclipse.jetty.util.StringUtil;
 import org.eclipse.jetty.util.TypeUtil;
 import org.eclipse.jetty.util.annotation.Name;
 import org.eclipse.jetty.util.log.Log;
@@ -49,11 +53,10 @@ import org.eclipse.jetty.util.ssl.X509;
 public class SecureRequestCustomizer implements HttpConfiguration.Customizer
 {
     private static final Logger LOG = Log.getLogger(SecureRequestCustomizer.class);
-
-    /**
-     * The name of the SSLSession attribute that will contain any cached information.
-     */
-    public static final String CACHED_INFO_ATTR = CachedInfo.class.getName();
+    public static final String JAVAX_SERVLET_REQUEST_X_509_CERTIFICATE = "javax.servlet.request.X509Certificate";
+    public static final String JAVAX_SERVLET_REQUEST_CIPHER_SUITE = "javax.servlet.request.cipher_suite";
+    public static final String JAVAX_SERVLET_REQUEST_KEY_SIZE = "javax.servlet.request.key_size";
+    public static final String JAVAX_SERVLET_REQUEST_SSL_SESSION_ID = "javax.servlet.request.ssl_session_id";
 
     private String sslSessionAttribute = "org.eclipse.jetty.servlet.request.ssl_session";
 
@@ -262,75 +265,33 @@ public class SecureRequestCustomizer implements HttpConfiguration.Customizer
 
         if (_sniHostCheck || _sniRequired)
         {
-            String name = request.getServerName();
             X509 x509 = (X509)sslSession.getValue(SniX509ExtendedKeyManager.SNI_X509);
-
             if (LOG.isDebugEnabled())
-                LOG.debug("Host {} with SNI {}", name, x509);
+                LOG.debug("Host {} with SNI {}", request.getServerName(), x509);
 
             if (x509 == null)
             {
                 if (_sniRequired)
                     throw new BadMessageException(400, "SNI required");
             }
-            else if (_sniHostCheck && !x509.matches(name))
+            else if (_sniHostCheck && !x509.matches(request.getServerName()))
             {
                 throw new BadMessageException(400, "Host does not match SNI");
             }
         }
 
-        try
-        {
-            String cipherSuite = sslSession.getCipherSuite();
-            Integer keySize;
-            X509Certificate[] certs;
-            String idStr;
-
-            CachedInfo cachedInfo = (CachedInfo)sslSession.getValue(CACHED_INFO_ATTR);
-            if (cachedInfo != null)
-            {
-                keySize = cachedInfo.getKeySize();
-                certs = cachedInfo.getCerts();
-                idStr = cachedInfo.getIdStr();
-            }
-            else
-            {
-                keySize = SslContextFactory.deduceKeyLength(cipherSuite);
-                certs = getCertChain(request, sslSession);
-                byte[] bytes = sslSession.getId();
-                idStr = TypeUtil.toHexString(bytes);
-                cachedInfo = new CachedInfo(keySize, certs, idStr);
-                sslSession.putValue(CACHED_INFO_ATTR, cachedInfo);
-            }
-
-            if (certs != null)
-                request.setAttribute("javax.servlet.request.X509Certificate", certs);
-
-            request.setAttribute("javax.servlet.request.cipher_suite", cipherSuite);
-            request.setAttribute("javax.servlet.request.key_size", keySize);
-            request.setAttribute("javax.servlet.request.ssl_session_id", idStr);
-            String sessionAttribute = getSslSessionAttribute();
-            if (sessionAttribute != null && !sessionAttribute.isEmpty())
-                request.setAttribute(sessionAttribute, sslSession);
-        }
-        catch (Exception e)
-        {
-            LOG.warn(Log.EXCEPTION, e);
-        }
+        request.setAttributes(new SslAttributes(request, sslSession, request.getAttributes()));
     }
 
-    private X509Certificate[] getCertChain(Request request, SSLSession sslSession)
+    private X509Certificate[] getCertChain(Connector connector, SSLSession sslSession)
     {
         // The in-use SslContextFactory should be present in the Connector's SslConnectionFactory
-        Connector connector = request.getHttpChannel().getConnector();
         SslConnectionFactory sslConnectionFactory = connector.getConnectionFactory(SslConnectionFactory.class);
         if (sslConnectionFactory != null)
         {
             SslContextFactory sslContextFactory = sslConnectionFactory.getSslContextFactory();
-            if (sslConnectionFactory != null)
-            {
+            if (sslContextFactory != null)
                 return sslContextFactory.getX509CertChain(sslSession);
-            }
         }
 
         // Fallback, either no SslConnectionFactory or no SslContextFactory instance found
@@ -353,34 +314,135 @@ public class SecureRequestCustomizer implements HttpConfiguration.Customizer
         return String.format("%s@%x", this.getClass().getSimpleName(), hashCode());
     }
 
+    private class SslAttributes extends Attributes.Wrapper
+    {
+        private final Request _request;
+        private final SSLSession _session;
+
+        private X509Certificate[] _certs;
+        private String _cipherSuite;
+        private Integer _keySize;
+        private String _sessionId;
+        private String _sessionAttribute;
+
+        public SslAttributes(Request request, SSLSession sslSession, Attributes attributes)
+        {
+            super(attributes);
+            this._request = request;
+            this._session = sslSession;
+
+            try
+            {
+                SslSessionData sslSessionData = getSslSessionData();
+                _certs = sslSessionData.getCerts();
+                _cipherSuite = _session.getCipherSuite();
+                _keySize = sslSessionData.getKeySize();
+                _sessionId = sslSessionData.getIdStr();
+                _sessionAttribute = getSslSessionAttribute();
+            }
+            catch (Exception e)
+            {
+                LOG.warn("Unable to get secure details ", e);
+            }
+        }
+
+        @Override
+        public Object getAttribute(String name)
+        {
+            switch (name)
+            {
+                case JAVAX_SERVLET_REQUEST_X_509_CERTIFICATE:
+                    return _certs;
+                case JAVAX_SERVLET_REQUEST_CIPHER_SUITE:
+                    return _cipherSuite;
+                case JAVAX_SERVLET_REQUEST_KEY_SIZE:
+                    return _keySize;
+                case JAVAX_SERVLET_REQUEST_SSL_SESSION_ID:
+                    return _sessionId;
+                default:
+                    if (!StringUtil.isEmpty(_sessionAttribute) && _sessionAttribute.equals(name))
+                        return _session;
+            }
+
+            return _attributes.getAttribute(name);
+        }
+
+        /**
+         * Get data belonging to the {@link SSLSession}.
+         *
+         * @return the SslSessionData
+         */
+        private SslSessionData getSslSessionData()
+        {
+            String key = SslSessionData.class.getName();
+            SslSessionData sslSessionData = (SslSessionData)_session.getValue(key);
+            if (sslSessionData == null)
+            {
+                String cipherSuite = _session.getCipherSuite();
+                int keySize = SslContextFactory.deduceKeyLength(cipherSuite);
+
+                X509Certificate[] certs = getCertChain(_request.getHttpChannel().getConnector(), _session);
+
+                byte[] bytes = _session.getId();
+                String idStr = TypeUtil.toHexString(bytes);
+
+                sslSessionData = new SslSessionData(keySize, certs, idStr);
+                _session.putValue(key, sslSessionData);
+            }
+            return sslSessionData;
+        }
+
+        @Override
+        public Set<String> getAttributeNameSet()
+        {
+            Set<String> names = new HashSet<>(_attributes.getAttributeNameSet());
+            names.remove(JAVAX_SERVLET_REQUEST_X_509_CERTIFICATE);
+            names.remove(JAVAX_SERVLET_REQUEST_CIPHER_SUITE);
+            names.remove(JAVAX_SERVLET_REQUEST_KEY_SIZE);
+            names.remove(JAVAX_SERVLET_REQUEST_SSL_SESSION_ID);
+
+            if (_certs != null)
+                names.add(JAVAX_SERVLET_REQUEST_X_509_CERTIFICATE);
+            if (_cipherSuite != null)
+                names.add(JAVAX_SERVLET_REQUEST_CIPHER_SUITE);
+            if (_keySize != null)
+                names.add(JAVAX_SERVLET_REQUEST_KEY_SIZE);
+            if (_sessionId != null)
+                names.add(JAVAX_SERVLET_REQUEST_SSL_SESSION_ID);
+            if (!StringUtil.isEmpty(_sessionAttribute))
+                names.add(_sessionAttribute);
+
+            return names;
+        }
+    }
+
     /**
-     * Simple bundle of information that is cached in the SSLSession. Stores the
-     * effective keySize and the client certificate chain.
+     * Simple bundle of data that is cached in the SSLSession.
      */
-    private static class CachedInfo
+    private static class SslSessionData
     {
-        private final X509Certificate[] _certs;
         private final Integer _keySize;
+        private final X509Certificate[] _certs;
         private final String _idStr;
 
-        CachedInfo(Integer keySize, X509Certificate[] certs, String idStr)
+        private SslSessionData(Integer keySize, X509Certificate[] certs, String idStr)
         {
             this._keySize = keySize;
             this._certs = certs;
             this._idStr = idStr;
         }
 
-        X509Certificate[] getCerts()
-        {
-            return _certs;
-        }
-
-        Integer getKeySize()
+        private Integer getKeySize()
         {
             return _keySize;
         }
 
-        String getIdStr()
+        private X509Certificate[] getCerts()
+        {
+            return _certs;
+        }
+
+        private String getIdStr()
         {
             return _idStr;
         }
