@@ -1,6 +1,6 @@
 //
 //  ========================================================================
-//  Copyright (c) 1995-2020 Mort Bay Consulting Pty Ltd and others.
+//  Copyright (c) 1995-2022 Mort Bay Consulting Pty Ltd and others.
 //  ------------------------------------------------------------------------
 //  All rights reserved. This program and the accompanying materials
 //  are made available under the terms of the Eclipse Public License v1.0
@@ -66,9 +66,11 @@ public class JDBCSessionDataStore extends AbstractSessionDataStore
     public static class SessionTableSchema
     {
         public static final int MAX_INTERVAL_NOT_SET = -999;
+        public static final String INFERRED = "INFERRED";
 
         protected DatabaseAdaptor _dbAdaptor;
         protected String _schemaName = null;
+        protected String _catalogName = null;
         protected String _tableName = "JettySessions";
         protected String _idColumn = "sessionId";
         protected String _contextPathColumn = "contextPath";
@@ -87,7 +89,20 @@ public class JDBCSessionDataStore extends AbstractSessionDataStore
         {
             _dbAdaptor = dbadaptor;
         }
+        
+        public void setCatalogName(String catalogName)
+        {
+            if (catalogName != null && StringUtil.isBlank(catalogName))
+                _catalogName = null;
+            else
+                _catalogName = catalogName;
+        }
 
+        public String getCatalogName()
+        {
+            return _catalogName;
+        }
+        
         public String getSchemaName()
         {
             return _schemaName;
@@ -95,8 +110,10 @@ public class JDBCSessionDataStore extends AbstractSessionDataStore
 
         public void setSchemaName(String schemaName)
         {
-            checkNotNull(schemaName);
-            _schemaName = schemaName;
+            if (schemaName != null && StringUtil.isBlank(schemaName))
+                _schemaName = null;
+            else
+                _schemaName = schemaName;
         }
 
         public String getTableName()
@@ -484,28 +501,48 @@ public class JDBCSessionDataStore extends AbstractSessionDataStore
 
                 //make the session table if necessary
                 String tableName = _dbAdaptor.convertIdentifier(getTableName());
+                
                 String schemaName = _dbAdaptor.convertIdentifier(getSchemaName());
-                try (ResultSet result = metaData.getTables(null, schemaName, tableName, null))
+                if (INFERRED.equalsIgnoreCase(schemaName))
+                {
+                    //use the value from the connection -
+                    //NOTE that this value will also now be prepended to ALL
+                    //table names in queries/updates.
+                    schemaName = connection.getSchema();
+                    setSchemaName(schemaName);
+                }
+                String catalogName = _dbAdaptor.convertIdentifier(getCatalogName());
+                if (INFERRED.equalsIgnoreCase(catalogName))
+                {
+                    //use the value from the connection
+                    catalogName = connection.getCatalog();
+                    setCatalogName(catalogName);
+                }
+                
+                try (ResultSet result = metaData.getTables(catalogName, schemaName, tableName, null))
                 {
                     if (!result.next())
                     {
+                        if (LOG.isDebugEnabled())
+                            LOG.debug("Creating table {} schema={} catalog={}", tableName, schemaName, catalogName);
                         //table does not exist, so create it
                         statement.executeUpdate(getCreateStatementAsString());
                     }
                     else
                     {
+                        if (LOG.isDebugEnabled())
+                            LOG.debug("Not creating table {} schema={} catalog={}", tableName, schemaName, catalogName);
                         //session table exists, check it has maxinterval column
                         ResultSet colResult = null;
                         try
                         {
-                            colResult = metaData.getColumns(null, schemaName, tableName,
+                            colResult = metaData.getColumns(catalogName, schemaName, tableName,
                                 _dbAdaptor.convertIdentifier(getMaxIntervalColumn()));
                         }
                         catch (SQLException sqlEx)
                         {
-                            LOG.warn("Problem checking if " + getTableName() +
-                                " table contains " + getMaxIntervalColumn() + " column. Ensure table contains column definition: \"" +
-                                getMaxIntervalColumn() + " long not null default -999\"");
+                            LOG.warn("Problem checking if {} table contains {} column. Ensure table contains column with definition: long not null default -999",
+                                getTableName(), getMaxIntervalColumn());
                             throw sqlEx;
                         }
                         try
@@ -519,9 +556,7 @@ public class JDBCSessionDataStore extends AbstractSessionDataStore
                                 }
                                 catch (SQLException sqlEx)
                                 {
-                                    LOG.warn("Problem adding " + getMaxIntervalColumn() +
-                                        " column. Ensure table contains column definition: \"" + getMaxIntervalColumn() +
-                                        " long not null default -999\"");
+                                    LOG.warn("Problem adding {} column. Ensure table contains column definition: long not null default -999", getMaxIntervalColumn());
                                     throw sqlEx;
                                 }
                             }
@@ -538,7 +573,7 @@ public class JDBCSessionDataStore extends AbstractSessionDataStore
 
                 boolean index1Exists = false;
                 boolean index2Exists = false;
-                try (ResultSet result = metaData.getIndexInfo(null, schemaName, tableName, false, true))
+                try (ResultSet result = metaData.getIndexInfo(catalogName, schemaName, tableName, false, true))
                 {
                     while (result.next())
                     {
@@ -559,8 +594,8 @@ public class JDBCSessionDataStore extends AbstractSessionDataStore
         @Override
         public String toString()
         {
-            return String.format("%s[%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s]", super.toString(),
-                _schemaName, _tableName, _idColumn, _contextPathColumn, _virtualHostColumn, _cookieTimeColumn, _createTimeColumn,
+            return String.format("%s[%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s]", super.toString(),
+                _catalogName, _schemaName, _tableName, _idColumn, _contextPathColumn, _virtualHostColumn, _cookieTimeColumn, _createTimeColumn,
                 _expiryTimeColumn, _accessTimeColumn, _lastAccessTimeColumn, _lastNodeColumn, _lastSavedTimeColumn, _maxIntervalColumn);
         }
     }
@@ -700,30 +735,24 @@ public class JDBCSessionDataStore extends AbstractSessionDataStore
                 statement.setString(2, cp); //context path
 
                 statement.setString(3, _context.getVhost()); //first vhost
-                statement.setString(4, data.getLastNode());//my node id
-                statement.setLong(5, data.getAccessed());//accessTime
+                statement.setString(4, data.getLastNode()); //my node id
+                statement.setLong(5, data.getAccessed()); //accessTime
                 statement.setLong(6, data.getLastAccessed()); //lastAccessTime
                 statement.setLong(7, data.getCreated()); //time created
-                statement.setLong(8, data.getCookieSet());//time cookie was set
+                statement.setLong(8, data.getCookieSet()); //time cookie was set
                 statement.setLong(9, data.getLastSaved()); //last saved time
                 statement.setLong(10, data.getExpiry());
                 statement.setLong(11, data.getMaxInactiveMs());
 
-                if (!data.getAllAttributes().isEmpty())
+                try (ByteArrayOutputStream baos = new ByteArrayOutputStream();
+                     ObjectOutputStream oos = new ObjectOutputStream(baos))
                 {
-                    try (ByteArrayOutputStream baos = new ByteArrayOutputStream();
-                         ObjectOutputStream oos = new ObjectOutputStream(baos))
-                    {
-                        SessionData.serializeAttributes(data, oos);
-                        byte[] bytes = baos.toByteArray();
-                        ByteArrayInputStream bais = new ByteArrayInputStream(bytes);
-                        statement.setBinaryStream(12, bais, bytes.length);//attribute map as blob
-                    }
-                }
-                else
-                {
-                    statement.setBinaryStream(12, EMPTY, 0);
+                    SessionData.serializeAttributes(data, oos);
+                    byte[] bytes = baos.toByteArray();
+                    ByteArrayInputStream bais = new ByteArrayInputStream(bytes);
+                    statement.setBinaryStream(12, bais, bytes.length); //attribute map as blob
                 }
+
                 statement.executeUpdate();
                 if (LOG.isDebugEnabled())
                     LOG.debug("Inserted session " + data);
@@ -739,30 +768,24 @@ public class JDBCSessionDataStore extends AbstractSessionDataStore
             connection.setAutoCommit(true);
             try (PreparedStatement statement = _sessionTableSchema.getUpdateSessionStatement(connection, data.getId(), _context))
             {
-                statement.setString(1, data.getLastNode());//should be my node id
-                statement.setLong(2, data.getAccessed());//accessTime
+                statement.setString(1, data.getLastNode()); //should be my node id
+                statement.setLong(2, data.getAccessed()); //accessTime
                 statement.setLong(3, data.getLastAccessed()); //lastAccessTime
                 statement.setLong(4, data.getLastSaved()); //last saved time
                 statement.setLong(5, data.getExpiry());
                 statement.setLong(6, data.getMaxInactiveMs());
 
-                if (!data.getAllAttributes().isEmpty())
+                try (ByteArrayOutputStream baos = new ByteArrayOutputStream();
+                     ObjectOutputStream oos = new ObjectOutputStream(baos))
                 {
-                    try (ByteArrayOutputStream baos = new ByteArrayOutputStream();
-                         ObjectOutputStream oos = new ObjectOutputStream(baos))
+                    SessionData.serializeAttributes(data, oos);
+                    byte[] bytes = baos.toByteArray();
+                    try (ByteArrayInputStream bais = new ByteArrayInputStream(bytes))
                     {
-                        SessionData.serializeAttributes(data, oos);
-                        byte[] bytes = baos.toByteArray();
-                        try (ByteArrayInputStream bais = new ByteArrayInputStream(bytes))
-                        {
-                            statement.setBinaryStream(7, bais, bytes.length);//attribute map as blob
-                        }
+                        statement.setBinaryStream(7, bais, bytes.length); //attribute map as blob
                     }
                 }
-                else
-                {
-                    statement.setBinaryStream(7, EMPTY, 0);
-                }
+
                 statement.executeUpdate();
 
                 if (LOG.isDebugEnabled())
@@ -865,7 +888,8 @@ public class JDBCSessionDataStore extends AbstractSessionDataStore
                         }
                         catch (Exception e)
                         {
-                            LOG.warn("{} Problem checking if potentially expired session {} exists in db", _context.getWorkerName(), k, e);
+                            LOG.warn("{} Problem checking if potentially expired session {} exists in db", _context.getWorkerName(), k);
+                            LOG.warn(e);
                         }
                     }
                 }
