@@ -1,6 +1,6 @@
 //
 //  ========================================================================
-//  Copyright (c) 1995-2020 Mort Bay Consulting Pty Ltd and others.
+//  Copyright (c) 1995-2022 Mort Bay Consulting Pty Ltd and others.
 //  ------------------------------------------------------------------------
 //  All rights reserved. This program and the accompanying materials
 //  are made available under the terms of the Eclipse Public License v1.0
@@ -21,6 +21,7 @@ package org.eclipse.jetty.util;
 import java.io.Closeable;
 import java.io.IOException;
 import java.io.InterruptedIOException;
+import java.util.Objects;
 import java.util.concurrent.CancellationException;
 import java.util.concurrent.TimeUnit;
 import java.util.concurrent.TimeoutException;
@@ -29,70 +30,64 @@ import java.util.concurrent.locks.ReentrantLock;
 
 import org.eclipse.jetty.util.log.Log;
 import org.eclipse.jetty.util.log.Logger;
-import org.eclipse.jetty.util.thread.NonBlockingThread;
 
-
-/* ------------------------------------------------------------ */
-/** Provides a reusable BlockingCallback.
+/**
+ * Provides a reusable {@link Callback} that can block the thread
+ * while waiting to be completed.
+ * <p>
  * A typical usage pattern is:
  * <pre>
  * void someBlockingCall(Object... args) throws IOException
  * {
- *   try(Blocker blocker=sharedBlockingCallback.acquire())
- *   {
- *     someAsyncCall(args,blocker);
- *     blocker.block();
- *   }
+ *     try(Blocker blocker = sharedBlockingCallback.acquire())
+ *     {
+ *         someAsyncCall(args, blocker);
+ *         blocker.block();
+ *     }
  * }
  * </pre>
  */
 public class SharedBlockingCallback
 {
-    static final Logger LOG = Log.getLogger(SharedBlockingCallback.class);
+    private static final Logger LOG = Log.getLogger(SharedBlockingCallback.class);
 
-    final ReentrantLock _lock = new ReentrantLock();
-    final Condition _idle = _lock.newCondition();
-    final Condition _complete = _lock.newCondition();
+    private static final Throwable IDLE = new ConstantThrowable("IDLE");
+    private static final Throwable SUCCEEDED = new ConstantThrowable("SUCCEEDED");
 
+    private static final Throwable FAILED = new ConstantThrowable("FAILED");
 
-    private static Throwable IDLE = new ConstantThrowable("IDLE");
+    private final ReentrantLock _lock = new ReentrantLock();
+    private final Condition _idle = _lock.newCondition();
+    private final Condition _complete = _lock.newCondition();
+    private Blocker _blocker = new Blocker();
 
-    private static Throwable SUCCEEDED = new ConstantThrowable("SUCCEEDED");
-
-    private static Throwable FAILED = new ConstantThrowable("FAILED");
-
-    Blocker _blocker;
-    
-    public SharedBlockingCallback()
-    {
-        _blocker=new Blocker();
-    }
-    
+    @Deprecated
     protected long getIdleTimeout()
     {
         return -1;
     }
-    
+
     public Blocker acquire() throws IOException
     {
-        _lock.lock();
         long idle = getIdleTimeout();
+        _lock.lock();
         try
         {
             while (_blocker._state != IDLE)
             {
-                if (idle>0 && (idle < Long.MAX_VALUE/2))
+                if (idle > 0 && (idle < Long.MAX_VALUE / 2))
                 {
                     // Wait a little bit longer than the blocker might block
-                    if (!_idle.await(idle*2,TimeUnit.MILLISECONDS))
+                    if (!_idle.await(idle * 2, TimeUnit.MILLISECONDS))
                         throw new IOException(new TimeoutException());
                 }
                 else
                     _idle.await();
             }
             _blocker._state = null;
+            return _blocker;
         }
-        catch (final InterruptedException e)
+        catch (InterruptedException x)
         {
             throw new InterruptedIOException();
         }
@@ -100,28 +95,56 @@ public class SharedBlockingCallback
         {
             _lock.unlock();
         }
-        return _blocker;
+    }
+
+    public boolean fail(Throwable cause)
+    {
+        Objects.requireNonNull(cause);
+        _lock.lock();
+        try
+        {
+            if (_blocker._state == null)
+            {
+                _blocker._state = new BlockerFailedException(cause);
+                _complete.signalAll();
+                return true;
+            }
+        }
+        finally
+        {
+            _lock.unlock();
+        }
+        return false;
     }
 
     protected void notComplete(Blocker blocker)
     {
-        LOG.warn("Blocker not complete {}",blocker);
+        LOG.warn("Blocker not complete {}", blocker);
         if (LOG.isDebugEnabled())
             LOG.debug(new Throwable());
     }
-    
-    /* ------------------------------------------------------------ */
-    /** A Closeable Callback.
+
+    /**
+     * A Closeable Callback.
      * Uses the auto close mechanism to check block has been called OK.
+     * <p>Implements {@link Callback} because calls to this
+     * callback do not blocak, rather they wakeup the thread that is blocked
+     * in {@link #block()}
      */
     public class Blocker implements Callback, Closeable
     {
-        Throwable _state = IDLE;
-        
+        private Throwable _state = IDLE;
+
         protected Blocker()
         {
         }
 
+        @Override
+        public InvocationType getInvocationType()
+        {
+            return InvocationType.NON_BLOCKING;
+        }
+
         @Override
         public void succeeded()
         {
@@ -134,7 +157,11 @@ public class SharedBlockingCallback
                     _complete.signalAll();
                 }
                 else
-                    throw new IllegalStateException(_state);
+                {
+                    LOG.warn("Succeeded after {}", _state.toString());
+                    if (LOG.isDebugEnabled())
+                        LOG.debug(_state);
+                }
             }
             finally
             {
@@ -150,17 +177,31 @@ public class SharedBlockingCallback
             {
                 if (_state == null)
                 {
-                    if (cause==null)
-                        _state=FAILED;
+                    if (cause == null)
+                        _state = FAILED;
                     else if (cause instanceof BlockerTimeoutException)
                         // Not this blockers timeout
-                        _state=new IOException(cause);
-                    else 
-                        _state=cause;
+                        _state = new IOException(cause);
+                    else
+                        _state = cause;
                     _complete.signalAll();
                 }
-                else 
-                    throw new IllegalStateException(_state);
+                else if (_state instanceof BlockerTimeoutException || _state instanceof BlockerFailedException)
+                {
+                    // Failure arrived late, block() already
+                    // modified the state, nothing more to do.
+                    if (LOG.isDebugEnabled())
+                        LOG.debug("Failed after {}", _state);
+                }
+                else
+                {
+                    LOG.warn("Failed after {}: {}", _state, cause);
+                    if (LOG.isDebugEnabled())
+                    {
+                        LOG.debug(_state);
+                        LOG.debug(cause);
+                    }
+                }
             }
             finally
             {
@@ -171,31 +212,34 @@ public class SharedBlockingCallback
         /**
          * Block until the Callback has succeeded or failed and after the return leave in the state to allow reuse. This is useful for code that wants to
          * repeatable use a FutureCallback to convert an asynchronous API to a blocking API.
-         * 
-         * @throws IOException
-         *             if exception was caught during blocking, or callback was cancelled
+         *
+         * @throws IOException if exception was caught during blocking, or callback was cancelled
          */
         public void block() throws IOException
         {
-            if (NonBlockingThread.isNonBlockingThread())
-                LOG.warn("Blocking a NonBlockingThread: ",new Throwable());
-            
-            _lock.lock();
             long idle = getIdleTimeout();
+            _lock.lock();
             try
             {
                 while (_state == null)
                 {
-                    if (idle>0 && (idle < Long.MAX_VALUE/2))
+                    if (idle > 0)
                     {
-                        // Wait a little bit longer than expected callback idle timeout
-                        if (!_complete.await(idle+idle/2,TimeUnit.MILLISECONDS))
-                            // The callback has not arrived in sufficient time.
-                            // We will synthesize a TimeoutException 
-                            _state=new BlockerTimeoutException();
+                        // Waiting here may compete with the idle timeout mechanism,
+                        // so here we wait a little bit longer to favor the normal
+                        // idle timeout mechanism that will call failed(Throwable).
+                        long excess = Math.min(idle / 2, 1000);
+                        if (!_complete.await(idle + excess, TimeUnit.MILLISECONDS))
+                        {
+                            // Method failed(Throwable) has not been called yet,
+                            // so we will synthesize a special TimeoutException.
+                            _state = new BlockerTimeoutException();
+                        }
                     }
                     else
+                    {
                         _complete.await();
+                    }
                 }
 
                 if (_state == SUCCEEDED)
@@ -214,6 +258,7 @@ public class SharedBlockingCallback
             }
             catch (final InterruptedException e)
             {
+                _state = e;
                 throw new InterruptedIOException();
             }
             finally
@@ -221,15 +266,12 @@ public class SharedBlockingCallback
                 _lock.unlock();
             }
         }
-        
+
         /**
          * Check the Callback has succeeded or failed and after the return leave in the state to allow reuse.
-         * 
-         * @throws IOException
-         *             if exception was caught during blocking, or callback was cancelled
          */
         @Override
-        public void close() throws IOException
+        public void close()
         {
             _lock.lock();
             try
@@ -241,19 +283,19 @@ public class SharedBlockingCallback
             }
             finally
             {
-                try 
+                try
                 {
-                    // If the blocker timed itself out, remember the state
-                    if (_state instanceof BlockerTimeoutException)
-                        // and create a new Blocker
-                        _blocker=new Blocker();
+                    // If we have a failure
+                    if (_state != null && _state != SUCCEEDED)
+                        // create a new Blocker
+                        _blocker = new Blocker();
                     else
                         // else reuse Blocker
                         _state = IDLE;
                     _idle.signalAll();
                     _complete.signalAll();
-                } 
-                finally 
+                }
+                finally
                 {
                     _lock.unlock();
                 }
@@ -266,7 +308,7 @@ public class SharedBlockingCallback
             _lock.lock();
             try
             {
-                return String.format("%s@%x{%s}",Blocker.class.getSimpleName(),hashCode(),_state);
+                return String.format("%s@%x{%s}", Blocker.class.getSimpleName(), hashCode(), _state);
             }
             finally
             {
@@ -274,8 +316,16 @@ public class SharedBlockingCallback
             }
         }
     }
-    
+
     private static class BlockerTimeoutException extends TimeoutException
-    { 
+    {
+    }
+
+    private static class BlockerFailedException extends Exception
+    {
+        public BlockerFailedException(Throwable cause)
+        {
+            super(cause);
+        }
     }
 }
