@@ -1,6 +1,6 @@
 //
 //  ========================================================================
-//  Copyright (c) 1995-2021 Mort Bay Consulting Pty Ltd and others.
+//  Copyright (c) 1995-2022 Mort Bay Consulting Pty Ltd and others.
 //  ------------------------------------------------------------------------
 //  All rights reserved. This program and the accompanying materials
 //  are made available under the terms of the Eclipse Public License v1.0
@@ -26,6 +26,7 @@ import java.util.concurrent.LinkedBlockingDeque;
 import java.util.concurrent.TimeUnit;
 
 import org.eclipse.jetty.io.ByteBufferPool;
+import org.eclipse.jetty.io.NullByteBufferPool;
 import org.eclipse.jetty.util.BufferUtil;
 import org.eclipse.jetty.util.log.Log;
 import org.eclipse.jetty.util.log.Logger;
@@ -83,7 +84,7 @@ public class MessageInputStream extends InputStream implements MessageAppender
     {
         this.timeoutMs = timeoutMs;
         this.session = session;
-        this.bufferPool = (session instanceof WebSocketSession) ? ((WebSocketSession)session).getBufferPool() : null;
+        this.bufferPool = (session instanceof WebSocketSession) ? ((WebSocketSession)session).getBufferPool() : new NullByteBufferPool();
     }
 
     @Override
@@ -155,8 +156,20 @@ public class MessageInputStream extends InputStream implements MessageAppender
             if (remainingContent)
                 LOG.warn("MessageInputStream closed without fully consuming content {}", session);
 
-            state = State.CLOSED;
+
+            // Release any buffers taken from the pool.
+            if (activeBuffer != null && activeBuffer != EOF)
+                bufferPool.release(activeBuffer);
+
+            for (ByteBuffer buffer : buffers)
+            {
+                if (buffer != null && buffer != EOF)
+                    bufferPool.release(buffer);
+            }
+
+            activeBuffer = null;
             buffers.clear();
+            state = State.CLOSED;
             buffers.add(EOF);
         }
     }
@@ -207,7 +220,7 @@ public class MessageInputStream extends InputStream implements MessageAppender
             }
 
             // grab a fresh buffer
-            while (activeBuffer == null || !activeBuffer.hasRemaining())
+            while (activeBuffer == null)
             {
                 if (LOG.isDebugEnabled())
                     LOG.debug("Waiting {} ms to read", timeoutMs);
@@ -246,6 +259,11 @@ public class MessageInputStream extends InputStream implements MessageAppender
                 SuspendToken resume = null;
                 synchronized (this)
                 {
+                    // Release buffer back to pool.
+                    if (activeBuffer != null && activeBuffer != EOF)
+                        bufferPool.release(activeBuffer);
+                    activeBuffer = null;
+
                     switch (state)
                     {
                         case CLOSED:
@@ -309,11 +327,6 @@ public class MessageInputStream extends InputStream implements MessageAppender
 
     private ByteBuffer acquire(int capacity, boolean direct)
     {
-        ByteBuffer buffer;
-        if (bufferPool != null)
-            buffer = bufferPool.acquire(capacity, direct);
-        else
-            buffer = direct ? BufferUtil.allocateDirect(capacity) : BufferUtil.allocate(capacity);
-        return buffer;
+        return bufferPool.acquire(capacity, direct);
     }
 }
