@@ -1,6 +1,6 @@
 //
 //  ========================================================================
-//  Copyright (c) 1995-2021 Mort Bay Consulting Pty Ltd and others.
+//  Copyright (c) 1995-2022 Mort Bay Consulting Pty Ltd and others.
 //  ------------------------------------------------------------------------
 //  All rights reserved. This program and the accompanying materials
 //  are made available under the terms of the Eclipse Public License v1.0
@@ -210,7 +210,7 @@ public class DistributionTests extends AbstractDistributionTest
     }
 
     @Test
-    @DisabledOnOs(OS.WINDOWS)  // jnr not supported on windows
+    @DisabledOnOs(value = OS.WINDOWS, disabledReason = "jnr not supported on windows")
     public void testUnixSocket() throws Exception
     {
         Path tmpSockFile;
